@@ -1,9 +1,11 @@
 import subprocess
 import sys
+from pathlib import Path
 
 import pytest
+from _pytest.monkeypatch import MonkeyPatch
 
-# test for _argcomplete but not specific for any application
+# Test for _argcomplete but not specific for any application.
 
 
 def equal_with_bash(prefix, ffc, fc, out=None):
@@ -11,16 +13,16 @@ def equal_with_bash(prefix, ffc, fc, out=None):
     res_bash = set(fc(prefix))
     retval = set(res) == res_bash
     if out:
-        out.write("equal_with_bash({}) {} {}\n".format(prefix, retval, res))
+        out.write(f"equal_with_bash({prefix}) {retval} {res}\n")
         if not retval:
             out.write(" python - bash: %s\n" % (set(res) - res_bash))
             out.write(" bash - python: %s\n" % (res_bash - set(res)))
     return retval
 
 
-# copied from argcomplete.completers as import from there
-# also pulls in argcomplete.__init__ which opens filedescriptor 9
-# this gives an OSError at the end of testrun
+# Copied from argcomplete.completers as import from there.
+# Also pulls in argcomplete.__init__ which opens filedescriptor 9.
+# This gives an OSError at the end of testrun.
 
 
 def _wrapcall(*args, **kargs):
@@ -31,7 +33,7 @@ def _wrapcall(*args, **kargs):
 
 
 class FilesCompleter:
-    "File completer class, optionally takes a list of allowed extensions"
+    """File completer class, optionally takes a list of allowed extensions."""
 
     def __init__(self, allowednames=(), directories=True):
         # Fix if someone passes in a string instead of a list
@@ -45,26 +47,16 @@ class FilesCompleter:
         completion = []
         if self.allowednames:
             if self.directories:
-                files = _wrapcall(
-                    ["bash", "-c", "compgen -A directory -- '{p}'".format(p=prefix)]
-                )
+                files = _wrapcall(["bash", "-c", f"compgen -A directory -- '{prefix}'"])
                 completion += [f + "/" for f in files]
             for x in self.allowednames:
                 completion += _wrapcall(
-                    [
-                        "bash",
-                        "-c",
-                        "compgen -A file -X '!*.{0}' -- '{p}'".format(x, p=prefix),
-                    ]
+                    ["bash", "-c", f"compgen -A file -X '!*.{x}' -- '{prefix}'"]
                 )
         else:
-            completion += _wrapcall(
-                ["bash", "-c", "compgen -A file -- '{p}'".format(p=prefix)]
-            )
+            completion += _wrapcall(["bash", "-c", f"compgen -A file -- '{prefix}'"])
 
-            anticomp = _wrapcall(
-                ["bash", "-c", "compgen -A directory -- '{p}'".format(p=prefix)]
-            )
+            anticomp = _wrapcall(["bash", "-c", f"compgen -A directory -- '{prefix}'"])
 
             completion = list(set(completion) - set(anticomp))
 
@@ -75,25 +67,26 @@ class FilesCompleter:
 
 class TestArgComplete:
     @pytest.mark.skipif("sys.platform in ('win32', 'darwin')")
-    def test_compare_with_compgen(self, tmpdir):
+    def test_compare_with_compgen(
+        self, tmp_path: Path, monkeypatch: MonkeyPatch
+    ) -> None:
         from _pytest._argcomplete import FastFilesCompleter
 
         ffc = FastFilesCompleter()
         fc = FilesCompleter()
 
-        with tmpdir.as_cwd():
-            assert equal_with_bash("", ffc, fc, out=sys.stdout)
+        monkeypatch.chdir(tmp_path)
 
-            tmpdir.ensure("data")
+        assert equal_with_bash("", ffc, fc, out=sys.stdout)
 
-            for x in ["d", "data", "doesnotexist", ""]:
-                assert equal_with_bash(x, ffc, fc, out=sys.stdout)
+        tmp_path.cwd().joinpath("data").touch()
+
+        for x in ["d", "data", "doesnotexist", ""]:
+            assert equal_with_bash(x, ffc, fc, out=sys.stdout)
 
     @pytest.mark.skipif("sys.platform in ('win32', 'darwin')")
     def test_remove_dir_prefix(self):
-        """this is not compatible with compgen but it is with bash itself:
-        ls /usr/<TAB>
-        """
+        """This is not compatible with compgen but it is with bash itself: ls /usr/<TAB>."""
         from _pytest._argcomplete import FastFilesCompleter
 
         ffc = FastFilesCompleter()
