@@ -1,22 +1,26 @@
+import email.message
+import io
 from typing import List
 from typing import Union
 
 import pytest
+from _pytest.monkeypatch import MonkeyPatch
+from _pytest.pytester import Pytester
 
 
 class TestPasteCapture:
     @pytest.fixture
     def pastebinlist(self, monkeypatch, request) -> List[Union[str, bytes]]:
-        pastebinlist = []  # type: List[Union[str, bytes]]
+        pastebinlist: List[Union[str, bytes]] = []
         plugin = request.config.pluginmanager.getplugin("pastebin")
         monkeypatch.setattr(plugin, "create_new_paste", pastebinlist.append)
         return pastebinlist
 
-    def test_failed(self, testdir, pastebinlist):
-        testpath = testdir.makepyfile(
+    def test_failed(self, pytester: Pytester, pastebinlist) -> None:
+        testpath = pytester.makepyfile(
             """
             import pytest
-            def test_pass():
+            def test_pass() -> None:
                 pass
             def test_fail():
                 assert 0
@@ -24,16 +28,16 @@ class TestPasteCapture:
                 pytest.skip("")
         """
         )
-        reprec = testdir.inline_run(testpath, "--pastebin=failed")
+        reprec = pytester.inline_run(testpath, "--pastebin=failed")
         assert len(pastebinlist) == 1
         s = pastebinlist[0]
         assert s.find("def test_fail") != -1
         assert reprec.countoutcomes() == [1, 1, 1]
 
-    def test_all(self, testdir, pastebinlist):
+    def test_all(self, pytester: Pytester, pastebinlist) -> None:
         from _pytest.pytester import LineMatcher
 
-        testpath = testdir.makepyfile(
+        testpath = pytester.makepyfile(
             """
             import pytest
             def test_pass():
@@ -44,7 +48,7 @@ class TestPasteCapture:
                 pytest.skip("")
         """
         )
-        reprec = testdir.inline_run(testpath, "--pastebin=all", "-v")
+        reprec = pytester.inline_run(testpath, "--pastebin=all", "-v")
         assert reprec.countoutcomes() == [1, 1, 1]
         assert len(pastebinlist) == 1
         contents = pastebinlist[0].decode("utf-8")
@@ -58,17 +62,17 @@ class TestPasteCapture:
             ]
         )
 
-    def test_non_ascii_paste_text(self, testdir, pastebinlist):
+    def test_non_ascii_paste_text(self, pytester: Pytester, pastebinlist) -> None:
         """Make sure that text which contains non-ascii characters is pasted
         correctly. See #1219.
         """
-        testdir.makepyfile(
+        pytester.makepyfile(
             test_unicode="""\
             def test():
                 assert '☺' == 1
             """
         )
-        result = testdir.runpytest("--pastebin=all")
+        result = pytester.runpytest("--pastebin=all")
         expected_msg = "*assert '☺' == 1*"
         result.stdout.fnmatch_lines(
             [
@@ -86,10 +90,8 @@ class TestPaste:
         return request.config.pluginmanager.getplugin("pastebin")
 
     @pytest.fixture
-    def mocked_urlopen_fail(self, monkeypatch):
-        """
-        monkeypatch the actual urlopen call to emulate a HTTP Error 400
-        """
+    def mocked_urlopen_fail(self, monkeypatch: MonkeyPatch):
+        """Monkeypatch the actual urlopen call to emulate a HTTP Error 400."""
         calls = []
 
         import urllib.error
@@ -97,18 +99,18 @@ class TestPaste:
 
         def mocked(url, data):
             calls.append((url, data))
-            raise urllib.error.HTTPError(url, 400, "Bad request", None, None)
+            raise urllib.error.HTTPError(
+                url, 400, "Bad request", email.message.Message(), io.BytesIO()
+            )
 
         monkeypatch.setattr(urllib.request, "urlopen", mocked)
         return calls
 
     @pytest.fixture
-    def mocked_urlopen_invalid(self, monkeypatch):
-        """
-        monkeypatch the actual urlopen calls done by the internal plugin
+    def mocked_urlopen_invalid(self, monkeypatch: MonkeyPatch):
+        """Monkeypatch the actual urlopen calls done by the internal plugin
         function that connects to bpaste service, but return a url in an
-        unexpected format
-        """
+        unexpected format."""
         calls = []
 
         def mocked(url, data):
@@ -127,11 +129,9 @@ class TestPaste:
         return calls
 
     @pytest.fixture
-    def mocked_urlopen(self, monkeypatch):
-        """
-        monkeypatch the actual urlopen calls done by the internal plugin
-        function that connects to bpaste service.
-        """
+    def mocked_urlopen(self, monkeypatch: MonkeyPatch):
+        """Monkeypatch the actual urlopen calls done by the internal plugin
+        function that connects to bpaste service."""
         calls = []
 
         def mocked(url, data):
@@ -149,7 +149,7 @@ class TestPaste:
         monkeypatch.setattr(urllib.request, "urlopen", mocked)
         return calls
 
-    def test_pastebin_invalid_url(self, pastebin, mocked_urlopen_invalid):
+    def test_pastebin_invalid_url(self, pastebin, mocked_urlopen_invalid) -> None:
         result = pastebin.create_new_paste(b"full-paste-contents")
         assert (
             result
@@ -157,24 +157,24 @@ class TestPaste:
         )
         assert len(mocked_urlopen_invalid) == 1
 
-    def test_pastebin_http_error(self, pastebin, mocked_urlopen_fail):
+    def test_pastebin_http_error(self, pastebin, mocked_urlopen_fail) -> None:
         result = pastebin.create_new_paste(b"full-paste-contents")
         assert result == "bad response: HTTP Error 400: Bad request"
         assert len(mocked_urlopen_fail) == 1
 
-    def test_create_new_paste(self, pastebin, mocked_urlopen):
+    def test_create_new_paste(self, pastebin, mocked_urlopen) -> None:
         result = pastebin.create_new_paste(b"full-paste-contents")
-        assert result == "https://bpaste.net/show/3c0c6750bd"
+        assert result == "https://bpa.st/show/3c0c6750bd"
         assert len(mocked_urlopen) == 1
         url, data = mocked_urlopen[0]
         assert type(data) is bytes
         lexer = "text"
-        assert url == "https://bpaste.net"
+        assert url == "https://bpa.st"
         assert "lexer=%s" % lexer in data.decode()
         assert "code=full-paste-contents" in data.decode()
         assert "expiry=1week" in data.decode()
 
-    def test_create_new_paste_failure(self, pastebin, monkeypatch):
+    def test_create_new_paste_failure(self, pastebin, monkeypatch: MonkeyPatch) -> None:
         import io
         import urllib.request
 
